You were mentioned by <span class='fw-bold'>@{{ username }}</span> in a {{ highlight_type }} in view <a href='{{ url_for("main.view_dashboard", **{ "view_name": view_name}) }}'>{{ display_name }}</a>.
