export const JENKINS_ADVANCED_SETTINGS = {
    gitRepo: {
        label: "Git Repository",
        help: "Repository link to a Github repo."
    },
    gitBranch: {
        label: "Git Branch",
        help: "Branch to build the job from."
    },
    pipelineFile: {
        label: "Pipeline file",
        help: "Path inside repository to the pipeline (.jenkinsfile) of the job."
    }
};
