{% extends "base.html.j2" %}

{% block javascripts %}
{{super()}}
<script defer type="module" src="/s/dist/testRuns.bundle.js"></script>
{% endblock javascripts %}

{% block title %}
Test Runs
{% endblock %}

{% block body %}
<div class="container mt-3">
<div class="row">
    <div class="col border rounded shadow-sm bg-main" id="testRunsBreakout">

    </div>
</div>
</div>
{% endblock %}
