{% extends "base.html.j2" %}

{% block title %}
Release Calendar for {{ release_name }}
{% endblock %}

{% block javascripts %}
{{ super() }}
<script>
    const releaseData = JSON.parse('{{ data | tojson | safe }}');
</script>
<script defer type="module" src="/s/dist/releaseScheduler.bundle.js"></script>
{% endblock javascripts %}

{% block body %}
    <div class="container-fluid">
        <div id="releaseScheduler"></div>
    </div>
{% endblock %}
