<div>You were assigned by <span style="font-weight:bold">@{{ username }}</span> to <a href='{{ config["BASE_URL"] }}{{url_for("main.runs", **{ "test_id": test_id, "additionalRuns[]": run_id })}}'>{{ build_id }}#{{ build_number }}</a>. </div>
