{% extends 'base.html.j2' %}

{% block title %}
  Login
{% endblock %}

{% block javascripts %}
{{ super() }}
<script defer type="module" src="/s/dist/login.bundle.js"></script>
<script>
  const GITHUB_SCOPES = "{{ github_scopes }}";
  const GITHUB_CLIENT_ID = "{{ github_cid }}";
  const CSRF_TOKEN = "{{ csrf_token }}";
  const LOGIN_METHODS = {{ config["LOGIN_METHODS"] or [] | tojson }};
</script>
{% endblock javascripts %}

{% block user_header %}
<a href="{{ url_for('main.home') }}">Home</a>
{% endblock user_header %}

{% block body %}
<div id="loginContainer">
</div>
{% endblock %}
