{% extends "base.html.j2" %}

{% block title %}
Teams
{% endblock %}

{% block javascripts %}
{{super()}}
<script defer type="module" src="/s/dist/teams.bundle.js"></script>
{% endblock javascripts %}

{% block body %}
{% if g.user %}
<div class="container-fluid bg-white rounded shadow-sm min-vh-100 my-2" id="teamsContainer">

</div>
{% else %}
{% endif %}
{% endblock %}
