html {
    height: 100%;
}

body {
    font-family: sans-serif;
    height: 100%;
    font-size: 14pt;
}

.logo {
    height: 64px;
}

main {
    flex: 1 0 auto;
}

footer {
    flex-shrink: 0;
}

.refresh, .refresh:visited, .refresh:hover {
    text-decoration: none;
    color: white;
}
