You were assigned by <span class='fw-bold'>@{{ username }}</span> to View Action <a
        href='{{ url_for("main.view_dashboard", **{ "view_name": view_name}) }}'>{{ display_name }}</a>.
<p>Action: {{ action }}</p>
