{% extends "base.html.j2" %}

{% block title %}
{{type}} Error
{% endblock %}

{% block body %}

<div class="container">
    <div class="row justify-content-center my-4">
        <div class="col text-center">
        <h1>ERROR - {{type}}</h1>
        </div>
    </div>
</div>

{% endblock %}
